@@ -1,77 +1,5 @@
 package Video::Xine;
 
-=pod
-
-=head1 NAME
-
-Video::Xine - Perl interface to libxine
-
-=head1 SYNOPSIS
-
-  use Video::Xine;
-
-  # Create and initialize the Xine object
-  my $xine = Video::Xine->new(
-    config_file => "$ENV{'HOME'}/.xine/config",
-  );
-
-  # Load a video driver
-  my $video_driver = Video::Xine::Driver::Video->new($xine,"auto",1,$x11_visual);
-
-  # Create a new stream (put your video driver under $DRIVER)
-  my $stream = $xine->stream_new(undef,$DRIVER);
-
-  # Open a file on the stream
-  $stream->open('file://my/movie/file.avi')
-    or die "Couldn't open stream: ", $stream->get_error();
-
-  # Get the current position (0 .. 65535), position in time, and length
-  # of stream in milliseconds
-  my ($pos, $pos_time, $length_time) = $stream->get_pos_length();
-
-  # Start the stream playing
-  $stream->play()
-     or die "Couldn't play stream: ", $xine->get_error();
-
-  # Play the stream to the end
-  while ( $stream->get_status() == XINE_STATUS_PLAY ) {
-    sleep(1);
-  }
-
-
-=head1 DESCRIPTION
-
-A perl interface to Xine, the Linux movie player. More properly, an
-interface to libxine, the development library. Requires installation of
-libxine.
-
-Xine by itself does not provide a user interface, and neither does
-this interface. Instead, you must set up the window using your own
-windowing code, and pass the window information to Xine.
-
-=head2 EXPORT
-
-=head3 STATUS CONSTANTS
-
-The Status constants are the results for a get_status() call. See
-xine.h for details.
-
-=over 4
-
-=item *
-
-XINE_STATUS_STOP
-
-Indicates that the stream is stopped.
-
-=item *
-
-XINE_STATUS_PLAY
-
-Indicates that the stream is playing.
-
-=cut
-
 use 5.008003;
 use strict;
 use warnings;
@@ -79,13 +7,51 @@ use warnings;
 use Exporter;
 use Carp;
 
-our $VERSION = '0.02';
+our $VERSION = '0.04';
 our @ISA = qw(Exporter);
 our @EXPORT = qw(
   XINE_STATUS_IDLE
   XINE_STATUS_STOP
   XINE_STATUS_PLAY
 
+  XINE_PARAM_SPEED                
+  XINE_PARAM_AV_OFFSET            
+  XINE_PARAM_AUDIO_CHANNEL_LOGICAL
+  XINE_PARAM_SPU_CHANNEL          
+  XINE_PARAM_VIDEO_CHANNEL        
+  XINE_PARAM_AUDIO_VOLUME         
+  XINE_PARAM_AUDIO_MUTE           
+  XINE_PARAM_AUDIO_COMPR_LEVEL    
+  XINE_PARAM_AUDIO_AMP_LEVEL      
+  XINE_PARAM_AUDIO_REPORT_LEVEL   
+  XINE_PARAM_VERBOSITY            
+  XINE_PARAM_SPU_OFFSET           
+  XINE_PARAM_IGNORE_VIDEO         
+  XINE_PARAM_IGNORE_AUDIO         
+  XINE_PARAM_IGNORE_SPU           
+  XINE_PARAM_BROADCASTER_PORT     
+  XINE_PARAM_METRONOM_PREBUFFER   
+  XINE_PARAM_EQ_30HZ              
+  XINE_PARAM_EQ_60HZ              
+  XINE_PARAM_EQ_125HZ             
+  XINE_PARAM_EQ_250HZ             
+  XINE_PARAM_EQ_500HZ             
+  XINE_PARAM_EQ_1000HZ            
+  XINE_PARAM_EQ_2000HZ            
+  XINE_PARAM_EQ_4000HZ            
+  XINE_PARAM_EQ_8000HZ            
+  XINE_PARAM_EQ_16000HZ           
+  XINE_PARAM_AUDIO_CLOSE_DEVICE   
+  XINE_PARAM_AUDIO_AMP_MUTE       
+  XINE_PARAM_FINE_SPEED           
+
+  XINE_SPEED_PAUSE 
+  XINE_SPEED_SLOW_4
+  XINE_SPEED_SLOW_2
+  XINE_SPEED_NORMAL
+  XINE_SPEED_FAST_2
+  XINE_SPEED_FAST_4
+
   XINE_EVENT_UI_PLAYBACK_FINISHED
   XINE_EVENT_UI_CHANNELS_CHANGED
   XINE_EVENT_UI_SET_TITLE
@@ -98,10 +64,23 @@ our @EXPORT = qw(
   XINE_EVENT_UI_NUM_BUTTONS
   XINE_EVENT_SPU_BUTTON
   XINE_EVENT_DROPPED_FRAMES
+  
+  XINE_ERROR_NONE
+  XINE_ERROR_NO_INPUT_PLUGIN
+  XINE_ERROR_NO_DEMUX_PLUGIN
+  XINE_ERROR_DEMUX_FAILED
+  XINE_ERROR_MALFORMED_MRL
+  XINE_ERROR_INPUT_FAILED
 
   XINE_GUI_SEND_DRAWABLE_CHANGED
   XINE_GUI_SEND_EXPOSE_EVENT
   XINE_GUI_SEND_VIDEOWIN_VISIBLE
+  
+  XINE_ENGINE_PARAM_VERBOSITY
+  
+  XINE_VERBOSITY_NONE
+  XINE_VERBOSITY_LOG
+  XINE_VERBOSITY_DEBUG  
 );
 
 require XSLoader;
@@ -109,26 +88,81 @@ XSLoader::load('Video::Xine', $VERSION);
 
 # Preloaded methods go here.
 
-use constant XINE_STATUS_IDLE => 0;
-use constant XINE_STATUS_STOP => 1;
-use constant XINE_STATUS_PLAY => 2;
+use constant {
+  XINE_STATUS_IDLE                  =>  0,
+  XINE_STATUS_STOP                  =>  1,
+  XINE_STATUS_PLAY                  =>  2,
 
-use constant XINE_EVENT_UI_PLAYBACK_FINISHED => 1;
-use constant XINE_EVENT_UI_CHANNELS_CHANGED => 2;
-use constant XINE_EVENT_UI_SET_TITLE => 3;
-use constant XINE_EVENT_UI_MESSAGE => 4;
-use constant XINE_EVENT_FRAME_FORMAT_CHANGE => 5;
-use constant XINE_EVENT_AUDIO_LEVEL => 6;
-use constant XINE_EVENT_QUIT => 7;
-use constant XINE_EVENT_PROGRESS => 8;
-use constant XINE_EVENT_MRL_REFERENCE => 9;
-use constant XINE_EVENT_UI_NUM_BUTTONS => 10;
-use constant XINE_EVENT_SPU_BUTTON => 11;
-use constant XINE_EVENT_DROPPED_FRAMES => 12;
+  XINE_PARAM_SPEED                  =>  1,
+  XINE_PARAM_AV_OFFSET              =>  2,
+  XINE_PARAM_AUDIO_CHANNEL_LOGICAL  =>  3,
+  XINE_PARAM_SPU_CHANNEL            =>  4,
+  XINE_PARAM_VIDEO_CHANNEL          =>  5,
+  XINE_PARAM_AUDIO_VOLUME           =>  6,
+  XINE_PARAM_AUDIO_MUTE             =>  7,
+  XINE_PARAM_AUDIO_COMPR_LEVEL      =>  8,
+  XINE_PARAM_AUDIO_AMP_LEVEL        =>  9,
+  XINE_PARAM_AUDIO_REPORT_LEVEL     => 10,
+  XINE_PARAM_VERBOSITY              => 11,
+  XINE_PARAM_SPU_OFFSET             => 12,
+  XINE_PARAM_IGNORE_VIDEO           => 13,
+  XINE_PARAM_IGNORE_AUDIO           => 14,
+  XINE_PARAM_IGNORE_SPU             => 15,
+  XINE_PARAM_BROADCASTER_PORT       => 16,
+  XINE_PARAM_METRONOM_PREBUFFER     => 17,
+  XINE_PARAM_EQ_30HZ                => 18,
+  XINE_PARAM_EQ_60HZ                => 19,
+  XINE_PARAM_EQ_125HZ               => 20,
+  XINE_PARAM_EQ_250HZ               => 21,
+  XINE_PARAM_EQ_500HZ               => 22,
+  XINE_PARAM_EQ_1000HZ              => 23,
+  XINE_PARAM_EQ_2000HZ              => 24,
+  XINE_PARAM_EQ_4000HZ              => 25,
+  XINE_PARAM_EQ_8000HZ              => 26,
+  XINE_PARAM_EQ_16000HZ             => 27,
+  XINE_PARAM_AUDIO_CLOSE_DEVICE     => 28,
+  XINE_PARAM_AUDIO_AMP_MUTE         => 29,
+  XINE_PARAM_FINE_SPEED             => 30,
 
-use constant XINE_GUI_SEND_DRAWABLE_CHANGED => 2;
-use constant XINE_GUI_SEND_EXPOSE_EVENT => 3;
-use constant XINE_GUI_SEND_VIDEOWIN_VISIBLE => 5;
+  XINE_SPEED_PAUSE                  =>  0,
+  XINE_SPEED_SLOW_4                 =>  1,
+  XINE_SPEED_SLOW_2                 =>  2,
+  XINE_SPEED_NORMAL                 =>  4,
+  XINE_SPEED_FAST_2                 =>  8,
+  XINE_SPEED_FAST_4                 => 16,
+
+  XINE_EVENT_UI_PLAYBACK_FINISHED   =>  1,
+  XINE_EVENT_UI_CHANNELS_CHANGED    =>  2,
+  XINE_EVENT_UI_SET_TITLE           =>  3,
+  XINE_EVENT_UI_MESSAGE             =>  4,
+  XINE_EVENT_FRAME_FORMAT_CHANGE    =>  5,
+  XINE_EVENT_AUDIO_LEVEL            =>  6,
+  XINE_EVENT_QUIT                   =>  7,
+  XINE_EVENT_PROGRESS               =>  8,
+  XINE_EVENT_MRL_REFERENCE          =>  9,
+  XINE_EVENT_UI_NUM_BUTTONS         => 10,
+  XINE_EVENT_SPU_BUTTON             => 11,
+  XINE_EVENT_DROPPED_FRAMES         => 12,
+  
+  XINE_ERROR_NONE                   =>  0,
+  XINE_ERROR_NO_INPUT_PLUGIN        =>  1,
+  XINE_ERROR_NO_DEMUX_PLUGIN        =>  2,
+  XINE_ERROR_DEMUX_FAILED           =>  3,
+  XINE_ERROR_MALFORMED_MRL          =>  4,
+  XINE_ERROR_INPUT_FAILED           =>  5,
+
+
+  XINE_GUI_SEND_DRAWABLE_CHANGED    =>  2,
+  XINE_GUI_SEND_EXPOSE_EVENT        =>  3,
+  XINE_GUI_SEND_VIDEOWIN_VISIBLE    =>  5,
+  
+  XINE_ENGINE_PARAM_VERBOSITY       =>  1,
+  
+  XINE_VERBOSITY_NONE               => 0,
+  XINE_VERBOSITY_LOG                => 1,
+  XINE_VERBOSITY_DEBUG              => 2
+  
+};
 
 sub new {
   my $type = shift;
@@ -262,6 +296,23 @@ sub get_status {
   return xine_get_status($self->{'stream'});
 }
 
+sub get_error {
+	my $self = shift;
+	return xine_get_error($self->{'stream'});
+}
+
+sub set_param {
+  my $self = shift;
+  my ($param, $value) = @_;
+  return xine_set_param($self->{'stream'}, $param, $value);
+}
+
+sub get_param {
+  my $self = shift;
+  my ($param) = @_;
+  return xine_get_param($self->{'stream'}, $param);
+}
+
 sub DESTROY {
   my $self = shift;
   xine_dispose($self->{'stream'});
@@ -292,6 +343,8 @@ sub new {
   my $type = shift;
   my ($xine, $id, $visual, $data) = @_;
 
+  $id ||= "auto";
+
   UNIVERSAL::isa($xine, 'Video::Xine')
       or croak "First argument must be of type Video::Xine (was $xine)";
 
@@ -299,7 +352,7 @@ sub new {
   $self->{'xine'} = $xine;
   if (scalar @_ > 1) {
     $self->{'driver'} = xine_open_video_driver($self->{'xine'}{'xine'},
-					       "auto",
+					       $id,
 					       $visual,
 					       $data
 					      )
@@ -357,7 +410,77 @@ sub get_event {
 1;
 __END__
 
-=pod
+=head1 NAME
+
+Video::Xine - Perl interface to libxine
+
+=head1 SYNOPSIS
+
+  use Video::Xine;
+
+  # Create and initialize the Xine object
+  my $xine = Video::Xine->new(
+    config_file => "$ENV{'HOME'}/.xine/config",
+  );
+
+  # Load a video driver
+  my $video_driver = Video::Xine::Driver::Video->new($xine,"auto",1,$x11_visual);
+
+  # Create a new stream (put your video driver under $DRIVER)
+  my $stream = $xine->stream_new(undef,$DRIVER);
+
+  # Open a file on the stream
+  $stream->open('file://my/movie/file.avi')
+    or die "Couldn't open stream: ", $stream->get_error();
+
+  # Get the current position (0 .. 65535), position in time, and length
+  # of stream in milliseconds
+  my ($pos, $pos_time, $length_time) = $stream->get_pos_length();
+
+  # Start the stream playing
+  $stream->play()
+     or die "Couldn't play stream: ", $xine->get_error();
+
+  # Play the stream to the end
+  while ( $xine->get_status() == XINE_STATUS_PLAY ) {
+    sleep(1);
+  }
+
+
+=head1 DESCRIPTION
+
+A perl interface to Xine, the Linux movie player. More properly, an
+interface to libxine, the development library. Requires installation of
+libxine.
+
+Xine by itself does not provide a user interface, and neither does
+this interface. Instead, you must set up the window using your own
+windowing code, and pass the window information to Xine.
+
+
+=head2 EXPORT
+
+=head3 STATUS CONSTANTS
+
+The Status constants are the results for a get_status() call. See
+xine.h for details.
+
+=over 4
+
+=item *
+
+XINE_STATUS_STOP
+
+Indicates that the stream is stopped.
+
+=item *
+
+XINE_STATUS_PLAY
+
+Indicates that the stream is playing.
+
+
+
 
 =head1 SEE ALSO
 
@@ -365,14 +488,18 @@ L<xine(1)>
 
 =head1 AUTHOR
 
-Stephen Nelson, E<lt>steven@localdomainE<gt>
+Stephen Nelson, E<lt>stephen@cpan.orgE<gt>
+
+=head1 SPECIAL THANKS TO
+
+Joern Reder for patches and GTK integration.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2005 by Stephen Nelson
+Copyright (C) 2005, 2006 by Stephen Nelson
 
 This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself, either Perl version 5.8.5 or,
+it under the same terms as Perl itself, either Perl version 5.8.6 or,
 at your option, any later version of Perl 5 you may have available.
 
 
